#!/bin/bash

######################################################################
#       FakeBTS.com
#       2017
#       v 0.1.11 (free version)
#######################################################################
#
#   Copyright (C) 2014 Pedro Cabrera
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Contact Info:
#    @PCabreraCamara
#    pedro.cabrera@fakebts.com
#
#######################################################################

# RUTAS de BINARIOS OSMOCOM
CELLBIN="/opt/osmocom-bb/src/host/layer23/src/misc/cell_log"
CCCHBIN="/opt/osmocom-bb/src/host/layer23/src/misc/ccch_scan"
L2SOCKET="/tmp/osmocom_l2"

# KALIBRATE
KALBIN_UHD="/opt/kalibrate-uhd/src/kal"
KALBIN_RTL="/opt/kalibrate-rtl/src/kal"

# AIRPROBE
GSMPATH="/opt/airprobe/gsm-receiver/src/python"
GP_UHD_BIN="gsm_receive_usrp.py"
GP_RTL_DEC="gsm_receive64-rtl.py"
GP_RTL_BIN="gsm_receive_rtl_3.7_console.py"
GP_BLADE_BIN="gsm_receive_hackrf_3.7.py"

# GRGSM
GRGSMCAP="/usr/local/bin/grgsm_capture.py"

# TSHARK
TSHARKBIN="/usr/local/bin/tshark"
ARFCNBIN="/opt/arfcncalc/arfcncalc"

# Application Parameter, Do not change if don't know what you are doing !!
# Device used
HARDWARE=""
LOOP="FALSE"
GPSMODE="FALSE"
# GSM900:	Banda de los 900Mhz 		(ARFCN 1-124)
# DSC:		Banda de los 1800Mhz		(ARFCN 512-885)
# EGSM:		Banda Ampliada de los 900Mhz	(ARFCN 975-1023)
#BANDAS="GSM900 DCS"
BANDAS=EGSM

# Output files
SCRIPTDIR="$( cd "$( /usr/bin/dirname "$0" )" && pwd )"
OUTDIR=$( echo "${SCRIPTDIR}/outputs")
EVENTDIR=$( echo "${SCRIPTDIR}/events")
TMPDIR=$( echo "${SCRIPTDIR}/tmp")
APPLOG=$( echo "${EVENTDIR}/app.log")
DEBUGLOG=$( echo "${EVENTDIR}/dbg.log")
ERRORLOG=$( echo "${EVENTDIR}/error.log")

# Timer per channel (broadcast traffic capture)
T1=30
# Timer ONLY for 2G Modems and OsmocomBB phones: time scanning cells (seconds):
T2=70

# GPS Parameters:
COORDINATES="00.000000, -00.000000"
## 	or point to GPS file with option "-g file" and then edit COORDINATES value:
#COORDINATES="/tmp/gps.txt"


# License

function disclaimer {
        echo "CellAnalysis  Copyright (C) 2014 Pedro Cabrera"
        echo "This program comes with ABSOLUTELY NO WARRANTY; for details visit http://www.gnu.org/licenses/gpl.txt"
        echo "This is free software, and you are welcome to redistribute it"
        echo "under certain conditions; for details visit http://www.gnu.org/licenses/gpl.txt."
        echo
}

# Help

function usage {
	echo "Usage: $0 {options: -w -l -g}"
	echo "   -w [OPTION]		Mandatory. Hardware used to analyze cells. Must be one of: \{modem, osmocombb, usrp, rtlsdr, rtlsdr-RPi or bladerf\}."
	echo
	echo "   -l			Optionally, run the application as infinite loop."
	echo
	echo "   -g			GPS mode, could be set in a variable or extract from file based on timestamp"
	echo
}

# Check folders
function check_folders {

	if [ ! -d "${OUTDIR}" ]; then
	  mkdir -p ${OUTDIR}
	fi

	if [ ! -d "${EVENTDIR}" ]; then
	  mkdir -p ${EVENTDIR}
	fi

	if [ ! -d "${TMPDIR}" ]; then
	  mkdir -p "${TMPDIR}"
	fi

	if [ ! -f "$OUTDIR/.cells_profiles" ]
	then
		touch $OUTDIR/.cells_profiles
	fi

	if [ -f "${SCRIPTDIR}/.lock_osmocom" ]
	then
		rm "${SCRIPTDIR}"/.lock_osmocom
	fi

	if [ -f "${SCRIPTDIR}/.lock_osmocom_global" ]
	then
		rm "${SCRIPTDIR}"/.lock_osmocom_global
	fi
	
	if [ -f "${SCRIPTDIR}/.lock_modem" ]
	then
		rm "${SCRIPTDIR}"/.lock_modem
	fi
}

# Check Binaries
function check_bin_osmocom {

			# Comprobamos que existen los binarios
			if [ ! -f "${CELLBIN}" ]
			then
				echo "Exit !!, ${CELLBIN} not found." | tee -a "${APPLOG}"
				exit 0
			fi

			if [ ! -f "${CCCHBIN}" ]
			then
				echo "Exit !!, ${CCCHBIN} not found." | tee -a "${APPLOG}"
				exit 0
			fi

			if [ ! -S "${L2SOCKET}" ]
			then
				echo "Exit !!, Socket ${L2SOCKET} not found." | tee -a "${APPLOG}"
				exit 0
			fi
}

function check_bin_usrp {
	
			# Comprobamos que existen los binarios
			if [ ! -f "${KALBIN_UHD}" ]
			then
				echo "Exit !!, ${KALBIN_UHD} not found." | tee -a "${APPLOG}"
				exit 0
			fi

			if [ ! -f "${GSMPATH}/${GP_UHD_BIN}" ]
			then
				echo "Exit !!, ${GSMPATH}/${GP_UHD_BIN} not found." | tee -a "${APPLOG}"
				exit 0
			fi
}

function check_bin_rtlsdr {

			# Comprobamos que existen los binarios
			if [ ! -f "${KALBIN_RTL}" ]
			then
				echo "Exit !!, ${KALBIN_RTL} not found." | tee -a "${APPLOG}"
				exit 0
			fi

			if [ ! -f "${GSMPATH}/${GP_RTL_DEC}" ]
			then
				echo "Exit !!, ${GSMPATH}/${GP_RTL_DEC} not found." | tee -a "${APPLOG}"
				exit 0
			fi
			
			if [ ! -f "${GRGSMCAP}" ]
			then
				echo "Exit !!, ${GRGSMCAP} not found." | tee -a "${APPLOG}"
				exit 0
			fi
}

function check_bin_bladerf {

			if [ ! -f "${GSMPATH}/${GP_BLADE_BIN}" ]
			then
				echo "Exit !!, ${GSMPATH}/${GP_BLADE_BIN} not found." | tee -a "${APPLOG}"
				exit 0
			fi

			if [ ! -f "${ARFCNBIN}" ]
			then
				echo "Exit !!, ${ARFCNBIN} not found." | tee -a "${APPLOG}"
				exit 0
			fi
}

function check_bin {

	check_folders

	case $1 in
        	"osmocombb")
			check_bin_osmocom
		;;
        	"usrp")
			check_bin_usrp
		;;
        	"rtlsdr")
			check_bin_rtlsdr
		;;
        	"bladerf")
			check_bin_bladerf
		;;
        esac

	if [ ! -f "${TSHARKBIN}" ]
	then
	        echo "Exit !!, ${TSHARKBIN} not found." | tee -a "${APPLOG}"
	        exit 0
	fi

	echo "OK --- Dependencies for $1 are all met."
}

function give_me_band {
#
# $1: ARFCN

if [ $# -eq 1 ]
then
	if [ $1 -ge 0 ] && [ $1 -le 124 ]
	then
		myband="GSM900"
	elif [ $1 -ge 512 ] && [ $1 -le 885 ]
	then
		myband="DCS"
	elif [ $1 -ge 975 ] && [ $1 -le 1023 ]
	then
		myband="EGSM"
	else
		myband="ERROR"
	fi

	echo "${myband}"
else
	echo "$(date '+%d/%m %H:%M:%S') Error, something wrong with parameters in give_me_band function. $# parameters: ${1}" >> ${ERRORLOG}
fi

}

function cal_num_subs {
#
# $1: Capture File
# $2: ARFCN
# $3: Optionally, modification for better filtering on osmocom devices

	if [ $# -eq 3 ]
	then
		filter="(gsmtap.arfcn == ${2}) && (gsm_a.dtap.msg_rr_type == 0x21)"
		arfcn=$2
	elif [ $# -eq 2 ]
	then
		filter="gsm_a.dtap.msg_rr_type == 0x21"
		arfcn=$2
	fi

	"${TSHARKBIN}" -r $1 -T pdml -2 -R "${filter}" > ${TMPDIR}/tshark_subs_${arfcn}.txt 2>&1

	num_paging=$(grep "MSI" ${TMPDIR}/tshark_subs_${arfcn}.txt | awk -F"Mobile Identity" '{print $1}' | awk -F"TMSI:" '{print $2}'| cut -d"\"" -f1 | sed '/^$/d' | sort | wc -l)
			
	if [ $num_paging -eq 0 ]
	then
		num_paging="Page_not_found"
	fi
	rm ${TMPDIR}/tshark_subs_${arfcn}.txt
			
	echo "${num_paging}"
}

function cal_num_ccch {
#
# $1: Capture File
# $2: ARFCN

	"${TSHARKBIN}" -z io,phs -r $1 > ${TMPDIR}/num_frames_$2.txt 2>&1
	grep ccch ${TMPDIR}/num_frames_$2.txt > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		num_lin=$(grep ccch ${TMPDIR}/num_frames_${arfcn}.txt | awk -F"frames:" '{print $2}'| cut -d" " -f1)
	else
		num_lin=0
	fi
	rm ${TMPDIR}/num_frames_$2.txt

	echo $num_lin
}

function out_chan_tmp {
#
# $1: capture file
# $2: ARFCN
#
	
	"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "gsm_a.dtap.msg_rr_type == 0x19" > ${TMPDIR}/tshark_canales_tmp_$2.txt 2>&1
	list_chan=$(cat ${TMPDIR}/tshark_canales_tmp_$2.txt |grep "List"| awk -F"\"" '{print $4}'| cut -d= -f2)
	echo "${list_chan}" > "${TMPDIR}/out_chan.tmp"
	rm ${TMPDIR}/tshark_canales_tmp_$2.txt
	echo "${TMPDIR}/out_chan.tmp"
}

function cal_num_chan {
#
# $1: Capture File
# $2: ARFCN
#
	if [ $# -eq 3 ]
	then
		filter="(gsmtap.arfcn == ${2}) && (gsm_a.dtap.msg_rr_type == 0x19)"
	elif [ $# -eq 2 ]
	then
		filter="gsm_a.dtap.msg_rr_type == 0x19"
	fi

	"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "${filter}" > ${TMPDIR}/tshark_canales_$2.txt 2>&1
	grep "List" ${TMPDIR}/tshark_canales_$2.txt > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		num_chan=$(cat ${TMPDIR}/tshark_canales_$2.txt |grep "List"| awk -F"\"" '{print $4}'| cut -d= -f2| awk '{print NF}')
		
		if [ ${num_chan} -eq 1 ]
		then
			channel=$(cat ${TMPDIR}/tshark_canales_$2.txt | grep "List"| awk -F"\"" '{print $4}'| cut -d= -f2)
		else
			channel=$(cat ${TMPDIR}/tshark_canales_$2.txt |grep "List"| awk -F"\"" '{print $4}'| cut -d= -f2)
		fi
	else
		num_chan="SI1_not_found"
		channel=NA
	fi
	rm ${TMPDIR}/tshark_canales_$2.txt

	echo "${num_chan}@${channel}"
}

function cal_cell_info {
#
# $1: Capture File
# $2: ARFCN

	if [ $# -eq 3 ]
	then
		filter="(gsmtap.arfcn == ${2}) && (gsm_a.dtap.msg_rr_type == 0x1b)"
	elif [ $# -eq 2 ]
	then
		filter="gsm_a.dtap.msg_rr_type == 0x1b"
	fi

	"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "${filter}" > ${TMPDIR}/tshark_radio_$2.txt 2>&1
	grep "System Information Type 3" ${TMPDIR}/tshark_radio_$2.txt > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		cellid=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "Cell CI" | awk -F"showname=" '{print $2}'| cut -d"\"" -f2|awk '{print $3}'| sed 's/0x//')
		lac=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "LAC" | awk -F"showname=" '{print $2}'| cut -d"\"" -f2|awk '{print $5}'| sed 's/0x//')
		mcc=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MCC"| awk -F"show=" '{print $2}'| cut -d"\"" -f2)
		mnc=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MNC"| awk -F"show=" '{print $2}'| cut -d"\"" -f2)
		operador=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MNC"| awk -F"showname=" '{print $2}'| cut -d"\"" -f2|cut -d":" -f2 | cut -d"(" -f1| sed 's/ /_/g')
			
	else

		if [ $# -eq 3 ]
		then
			filter="(gsmtap.arfcn == ${2}) && (gsm_a.dtap.msg_rr_type == 0x1c)"
		elif [ $# -eq 2 ]
		then
			filter="gsm_a.dtap.msg_rr_type == 0x1c"
		fi
		"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "${filter}" > ${TMPDIR}/tshark_radio_$2.txt 2>&1
		grep "System Information Type 4" ${TMPDIR}/tshark_radio_$2.txt > /dev/null 2>&1
		if [ $? -eq 0 ]
		then
			cellid="SI4_found"
			lac=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "LAC" | awk -F"showname=" '{print $2}'| cut -d"\"" -f2|awk '{print $5}'| sed 's/0x//')
			mcc=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MCC"| awk -F"show=" '{print $2}'| cut -d"\"" -f2)
			mnc=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MNC"| awk -F"show=" '{print $2}'| cut -d"\"" -f2)
			operador=$(cat ${TMPDIR}/tshark_radio_$2.txt |grep "MNC"| awk -F"showname=" '{print $2}'| cut -d"\"" -f2|cut -d":" -f2 | cut -d"(" -f1| sed 's/ /_/g')
			
		else
			cellid="SI3_not_found"
			lac="NA"
			mcc="NA"
			mnc="NA"
			operador="NA"
		fi

	fi
		
	rm ${TMPDIR}/tshark_radio_$2.txt

	echo "${cellid}@${lac}@${mcc}@${mnc}@${operador}"
}

###########################################################################################################
#
#	Phase 0: Find close cells
#
###########################################################################################################

function find_close_cells {
# 
# $1: Hardware

	if [ $1 != "osmocombb" ]
	then
		# Si exite una ejecucion en curso, me planto
		CHECK=$(ps -ef|grep -v "grep"|grep gsm_receive)
		if [ $? -eq 0 ]
		then
			echo "$(date '+%d/%m %H:%M:%S') Airprobe is already running." | tee -a "${APPLOG}"
			exit 0	
		fi		
	fi

	case $1 in
        	"usrp")
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells, This may take some time ..." | tee -a "${APPLOG}"
			echo > ${TMPDIR}/phase1.out
			for band in ${BANDAS}
			do
				${KALBIN_UHD} -g 40.0 -F 52e6 -s ${band} &>> ${TMPDIR}/phase1.out
			done
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells finished"
			;;
		"rtlsdr")
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells, This may take some time ..." | tee -a "${APPLOG}"
			echo > ${TMPDIR}/phase1.out
			for band in ${BANDAS}
			do
				${KALBIN_RTL} -s ${band} &>> ${TMPDIR}/phase1.out
			done
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells finished"
			;;
		"rtlsdr-RPi")
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells, This may take some time ..." | tee -a "${APPLOG}"
			echo > ${TMPDIR}/phase1.out
			for band in ${BANDAS}
			do
				${KALBIN_RTL} -s ${band} &>> ${TMPDIR}/phase1.out
			done
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells finished"
			;;
		"modem")
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells, This may take some time ..." | tee -a "${APPLOG}"	
			if [ ! -f ${SCRIPTDIR}/.lock_modem ]
			then
				touch "${SCRIPTDIR}"/.lock_modem
				stty -F /dev/ttyUSB0 115200 cs8 -ignpar -cstopb -icrnl -inlcr -ocrnl -onlcr -echo	
				echo -n -e "AT\r" > /dev/ttyUSB0
				sleep 2
				echo -n -e "AT+CENG=2,1\r" > /dev/ttyUSB0
				sleep 2
				echo -n -e "AT+CBAND=\"ALL_BAND\"\r" > /dev/ttyUSB0	
				sleep 2
			fi

			cat /dev/ttyUSB0 > ${TMPDIR}/phase1.tmp &
			echo $! > "${SCRIPTDIR}"/.lock_modem	
			echo -n -e "AT+COPS=?\r" > /dev/ttyUSB0
			breaktheloop=0
			while true
			do
				grep -e "COPS:" -e ERROR ${TMPDIR}/phase1.tmp > /dev/null 2>&1
				if [ $? -eq 0 ] || [ ${breaktheloop} -eq ${T2} ]
				then
					break
				fi

				breaktheloop=$(echo "${breaktheloop} + 1"|bc)
				sleep 1
			done	

			cat ${TMPDIR}/phase1.tmp | grep -a -v "65535"| grep -a -v -e "OK" -e "AT" -e "ERROR" | cut -d"," -f2,3 | sed 's/\"//' |  grep -o -E '^[[:alnum:]]{4}[[:punct:]][[:digit:]]{2}' | awk -F"," '{if($2>=40){print $1,$2}}' | sort | uniq > ${TMPDIR}/phase1.out
			disown
			kill -9 $(cat "${SCRIPTDIR}"/.lock_modem) > /dev/null 2>&1
			echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells finished"
			;;
		"osmocombb")
			if [ ! -f "${SCRIPTDIR}"/.lock_osmocom ]
			then
				touch "${SCRIPTDIR}"/.lock_osmocom
				osmoconpid=$(ps -ef|grep 'osmocon' | grep -v 'grep' | awk '{print $2}')
				if [ ${osmoconpid} -gt 0 ] > /dev/null 2>&1
				then
					echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells, This may take some time ..." | tee -a "${APPLOG}"
					echo > ${TMPDIR}/phase1.out
					echo > ${TMPDIR}/logfile.txt
					"${CELLBIN}" -l ${TMPDIR}/logfile.txt -n -s ${L2SOCKET} &>> ${TMPDIR}/phase1.out &
					sleep $T2
					disown
					kill -9 $!
					#	Line added due to DUAL behaviour
					#
					cat ${TMPDIR}/phase1.out | awk '{if($3=="Cell:"){printf "%s;%s;%s;%s;%s\n", $4, $5, $6, $7, $8 }}' | sed 's/ARFCN=//g' | sed 's/MCC=//g' | sed 's/MNC=//g' > ${TMPDIR}/arfcn_list.tmp

					echo "$(date '+%d/%m %H:%M:%S')  --- Looking for close cells finished"
	
					rm "${SCRIPTDIR}"/.lock_osmocom
				else
					echo "$(date '+%d/%m %H:%M:%S')  Error looking for close cells, Osmocon is not running." | tee -a "${APPLOG}"
				fi
			else
				sleep 30
			fi
			;;
	esac
	
}

###########################################################################################################
#
#	Phase 1: Scan traffic per cell
#
###########################################################################################################

function scan_cells {

	case $1 in
        	"osmocombb")
			scan_cells_osmocom
		;;
        	"usrp")
			scan_cells_usrp_rtl
		;;
        	"rtlsdr")
			scan_cells_usrp_rtl
		;;
			"rtlsdr-RPi")
			scan_cells_usrp_rtl
		;;
        	"bladerf")
			scan_cells_bladerf
		;;
			"modem")
			scan_cells_modem
		;;
        esac
}

function scan_cells_osmocom {


	if [ ! -f "${SCRIPTDIR}"/.lock_osmocom_global ]
	then

		touch "${SCRIPTDIR}"/.lock_osmocom_global
		cp ${TMPDIR}/arfcn_list.tmp ${TMPDIR}/arfcn_list.txt

		if [ $(wc -l ${TMPDIR}/arfcn_list.txt | cut -d" " -f1) -gt 0 ]
		then
	
			NUMCHAN=$(wc -l ${TMPDIR}/arfcn_list.txt | cut -d" " -f1)
			counter=1
			ignored=0

			cat ${TMPDIR}/arfcn_list.txt | while read linea
			do

				arfcn=$(echo "${linea}" | cut -d";" -f1)
				mcc=$(echo "${linea}" | cut -d";" -f2)
				mnc=$(echo "${linea}" | cut -d";" -f3)

				rxlevel=$(cat ${TMPDIR}/logfile.txt | awk -v arfcn=${arfcn} '{if(match($0,"sysinfo")){getline;if($0=="arfcn " arfcn){getline;getline;getline;print $0}}}' | cut -d" " -f2)

				echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- Processing arfcn: ${arfcn} rxlevel ${rxlevel}, (${counter} / ${NUMCHAN})" | tee -a "${APPLOG}"

				if [ ${DUAL} == "FALSE" ]
				then
				
					time=$(date '+%H:%M:%S')

					scan_channel ${HARDWARE} ${arfcn} "" ${time}

					sucess=$(extract_cell_data ${HARDWARE} ${TMPDIR}/tshark_${arfcn}_${time}.pcap ${arfcn} &)
				
					if [ ${sucess} -eq 0 ]
					then
						ignored=$(expr $ignored + 1)
						echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} ignored." | tee -a "${APPLOG}"
					else
						echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} saved." | tee -a "${APPLOG}"
					fi
				else

					if ([ $arfcn -ge 0 ] && [ $arfcn -le 124 ]) || ([ $arfcn -ge 975 ] && [ $arfcn -le 1023 ])
					then
						freq=$("${ARFCNBIN}" -a $arfcn -b 900 -d)
					else
						freq=$("${ARFCNBIN}" -a $arfcn -b 1800 -d)
					fi

					scan_channel ${HARDWARE2} ${arfcn} ${freq} ${time}

					extract_cell_data ${HARDWARE2} ${TMPDIR}/tshark_${arfcn}_${time}.pcap ${arfcn} ${TMPDIR}/log_airprobe_${arfcn}_${time}.log &
				fi

				counter=$(expr $counter + 1)
			done

			rm ${TMPDIR}/arfcn_list.txt

			if [ ${NUMCHAN} -eq ${ignored} ]
			then
				osmoconpid=$(ps -ef|grep 'osmocon' | grep -v 'grep' | awk '{print $2}')
			
				if [ -v ${osmoconpid} ]
				then
					echo "$(date '+%d/%m %H:%M:%S')  Error, all cells ignored and Osmocon is running: ${osmoconpid}." | tee -a "${APPLOG}"
				else
					echo "$(date '+%d/%m %H:%M:%S')  Error, all cells ignored and Osmocon is not running." | tee -a "${APPLOG}"
				fi
				
			fi

			echo "$(date '+%d/%m %H:%M:%S') OK --- Scan finished." | tee -a "${APPLOG}"

		else
			osmoconpid=$(ps -ef|grep 'osmocon' | grep -v 'grep' | awk '{print $2}')
			
			if [ ${osmoconpid} -gt 0 ] > /dev/null 2>&1 
			then
				echo "$(date '+%d/%m %H:%M:%S')  Error, not cells found. Is your mobile on?. Osmocon is running." | tee -a "${APPLOG}"
			else
				echo "$(date '+%d/%m %H:%M:%S')  Error, not cells found. Is your mobile on?. Osmocon is not running." | tee -a "${APPLOG}"
			fi
		fi

		rm "${SCRIPTDIR}"/.lock_osmocom_global

	else
		sleep 20
	fi
}

function scan_cells_usrp_rtl {

		grep "chan:" ${TMPDIR}/phase1.out > /dev/null 2>&1

		if [ $? -eq 0 ]
		then

			numchan=$(grep "chan:" ${TMPDIR}/phase1.out | wc -l)
			counter=1
			grep "chan:" ${TMPDIR}/phase1.out | while read linea
			do
				arfcn=$(echo "${linea}" | awk '{print $(NF-5)}')
				freq_tmp=$(echo "${linea}" | awk '{print $(NF-4)}' | sed 's/(//g'| sed 's/MHz//g' | sed 's/\.//g')
				freq_base=$(echo ${freq_tmp}00000)
				###############################################################################################
				# Warning:
				# Version 0.1.11:  New RTLSDR dongles have 0 ppm, so I ignore the frequency correction:
				#sign=$(echo "${linea}" | awk '{print $(NF-3)}')
				#offset=$(echo "${linea}" | awk '{print $(NF-2)}' | sed 's/)//g'| sed 's/kHz//g' | sed 's/\.//g')
				#freq_final=$(echo $[${freq_base} ${sign} ${offset}])
				##############################################################################################
				freq_final=$(echo ${freq_tmp}00000)
				power=$(echo "${linea}" | awk '{print $(NF)}')
				rxlevel=0

				date=$(date '+%d/%m')
				time=$(date '+%H:%M:%S')

				echo "${date} ${time}  --- Processing arfcn: ${arfcn}, (${counter} / ${numchan})" | tee -a "${APPLOG}"

				scan_channel ${HARDWARE} ${arfcn} ${freq_final} ${time}

				sucess=$(extract_cell_data ${HARDWARE} ${TMPDIR}/tshark_${arfcn}_${time}.pcap ${arfcn} ${TMPDIR}/log_airprobe_${arfcn}_${time}.log &)
				if [ ${sucess} -eq 0 ]
				then
					ignored=$(expr $ignored + 1)
					echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} ignored." | tee -a "${APPLOG}"
				else
					echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} saved." | tee -a "${APPLOG}"
				fi

				counter=$(expr $counter + 1)

			done	

			echo "$(date '+%d/%m %H:%M:%S') OK --- Scan finished." | tee -a "${APPLOG}"

		else
			echo "$(date '+%d/%m %H:%M:%S') Error, not cells found. Is your Usrp/RTL on?" | tee -a "${APPLOG}"
			echo "$(date '+%d/%m %H:%M:%S') No cells were found." >> ${ERRORLOG} | tee -a "${APPLOG}"
		fi
}

function scan_cells_modem {

		numchan=$( wc -l ${TMPDIR}/phase1.out | awk '{print $1}') > /dev/null 2>&1

		if [ $numchan -gt 0 ]
		then
			counter=1
			cat ${TMPDIR}/phase1.out | while read linea
			do
				arfcntmp=$(echo "${linea}" | cut -d"," -f1)
				rxlevel=$(echo "${linea}" | cut -d"," -f2)

				arfcn=$((10#$arfcntmp))

				if ([ $arfcn -ge 0 ] && [ $arfcn -le 124 ]) || ([ $arfcn -ge 975 ] && [ $arfcn -le 1023 ])
				then
						freq=$("${ARFCNBIN}" -a $arfcn -b 900 -d)
				else
						freq=$("${ARFCNBIN}" -a $arfcn -b 1800 -d)
				fi
				date=$(date '+%d/%m')
				time=$(date '+%H:%M:%S')

				echo "${date} ${time}  --- Processing arfcn: ${arfcn}, rxlevel ${rxlevel}, (${counter} / ${numchan})" | tee -a "${APPLOG}"

				scan_channel ${HARDWARE2} ${arfcn} ${freq} ${time}

				sucess=$(extract_cell_data ${HARDWARE2} ${TMPDIR}/tshark_${arfcn}_${time}.pcap ${arfcn} ${TMPDIR}/log_airprobe_${arfcn}_${time}.log &)
				
				if [ ${sucess} -eq 0 ]
				then
					ignored=$(expr $ignored + 1)
					echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} ignored." | tee -a "${APPLOG}"
				else
					echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- arfcn: ${arfcn} saved." | tee -a "${APPLOG}"
				fi

				counter=$(expr $counter + 1)

			done	

			echo "$(date '+%d/%m %H:%M:%S') OK --- Scan finished." | tee -a "${APPLOG}"

		else
			echo "$(date '+%d/%m %H:%M:%S') Error, not cells found. Check your modem" | tee -a "${APPLOG}"
			echo "$(date '+%d/%m %H:%M:%S') No cells were found." >> ${ERRORLOG} | tee -a "${APPLOG}"
		fi
}


function scan_cells_bladerf_band {

#	begining ARFCN: $1
#	ending ARFCN: $2

	INICIO=$1
	FIN=$2
	counter=1

	for (( i=$INICIO; i<=$FIN; i++ ))
	do
			NUMCHAN=$(expr $FIN - $INICIO)
			arfcn=$i

			if [ $BANDAS == "GSM900" ] || [ $BANDAS == "EGSM" ]
			then
				freq=$("${ARFCNBIN}" -a $i -b 900 -d)
			elif [ $BANDAS == "DCS" ]
			then
				freq=$("${ARFCNBIN}" -a $i -b 1800 -d)
			fi

			echo "$(date '+%d/%m %H:%M:%S')  --- Processing arfcn: ${arfcn}, (${counter} / ${NUMCHAN})" | tee -a "${APPLOG}"
			scan_channel ${HARDWARE} ${arfcn} ${freq}

			extract_cell_data ${HARDWARE} ${TMPDIR}/tshark_${arfcn}.pcap ${arfcn} &

			counter=$(expr $counter + 1)
	done
	
	echo "$(date '+%d/%m %H:%M:%S') OK --- Scan finished." | tee -a "${APPLOG}"
}

function scan_cells_bladerf {
	
	grep ";" ${TMPDIR}/arfcn_list.tmp > /dev/null 2>&1

		if [ $? -eq 0 ]
		then

			numchan=$(wc -l ${TMPDIR}/arfcn_list.tmp | awk '{print $1}')
			counter=1
			
			cat ${TMPDIR}/arfcn_list.tmp | while read linea
			do
				arfcn=$(echo "${linea}" | cut -d";" -f1)
				mcc=$(echo "${linea}" | cut -d";" -f2)
				mnc=$(echo "${linea}" | cut -d";" -f3)

				rxlevel=$(cat ${TMPDIR}/logfile.txt | awk -v arfcn=${arfcn} '{if(match($0,"sysinfo")){getline;if($0=="arfcn " arfcn){getline;getline;getline;print $0}}}' | cut -d" " -f2)

				echo "$(date '+%d/%m') $(date '+%H:%M:%S')  --- Processing arfcn: ${arfcn} rxlevel ${rxlevel}, (${counter} / ${numchan})" | tee -a "${APPLOG}"

				if ([ $arfcn -ge 0 ] && [ $arfcn -le 124 ]) || ([ $arfcn -ge 975 ] && [ $arfcn -le 1023 ])
				then
					freq=$("${ARFCNBIN}" -a $arfcn -b 900 -d)
				else
					freq=$("${ARFCNBIN}" -a $arfcn -b 1800 -d)
				fi

				scan_channel ${HARDWARE2} ${arfcn} ${freq} ${time}

				extract_cell_data ${HARDWARE2} ${TMPDIR}/tshark_${arfcn}_${time}.pcap ${arfcn} ${TMPDIR}/log_airprobe_${arfcn}_${time}.log

				counter=$(expr $counter + 1)

			done	

			echo "$(date '+%d/%m %H:%M:%S') OK --- Scan finished." | tee -a "${APPLOG}"

		else
			echo "$(date '+%d/%m %H:%M:%S') Error, not cells found. Is your bladerf on?" | tee -a "${APPLOG}"
			echo "$(date '+%d/%m %H:%M:%S') No cells were found." >> ${ERRORLOG} | tee -a "${APPLOG}"
		fi
}

###########################################################################################################
#
#	Phase 1.1: Generate PCAP file for a given frequency
#
###########################################################################################################

function scan_channel {
#
# $1: Hardware
# $2: ARFCN
# $3: frequency
# $4: Time
#

	case $1 in
        "usrp")
			"${TSHARKBIN}" -i lo -a duration:${T1} -w ${TMPDIR}/tshark_${2}_${4}.pcap > /dev/null 2>&1 &
			cd ${GSMPATH} > /dev/null 2>&1
			./${GP_UHD_BIN} -f ${3} -d 174 -c 0B -g 40  > ${TMPDIR}/log_airprobe_${2}_${4}.log 2>&1 &
			sleep ${T1}
			disown
			kill -9 $! > /dev/null 2>&1
			cd - > /dev/null 2>&1
			;;
		"rtlsdr")
			"${TSHARKBIN}" -i lo -a duration:${T1} -w ${TMPDIR}/tshark_${2}_${4}.pcap > /dev/null 2>&1 &
			cd ${GSMPATH} > /dev/null 2>&1
			./${GP_RTL_BIN} -s 1e6 -f ${3} -g 42 > ${TMPDIR}/log_airprobe_${2}_${4}.log 2>&1 &
			sleep $T1
			disown
			kill -9 $! > /dev/null 2>&1
			cd - > /dev/null 2>&1
			;;
		"rtlsdr-RPi")
			# Version Rpi3
			# Due to CPU capture is splited in 2 phases; capture to file and decode file recorded
			#
			"${GRGSMCAP}" -a ${2} -g 42 -T ${T1} -c ${TMPDIR}/capture_${2}_${4}.cfile > /dev/null 2>&1
			"${TSHARKBIN}" -i lo -w ${TMPDIR}/tshark_${2}_${4}.pcap > /dev/null 2>&1 &
			cd ${GSMPATH} > /dev/null 2>&1
			./${GP_RTL_DEC} -I ${TMPDIR}/capture_${2}_${4}.cfile -d 64 -c 0B > ${TMPDIR}/log_airprobe_${2}_${4}.log 2>&1
			disown
			kill -9 $! > /dev/null
			rm ${TMPDIR}/capture_${2}_${4}.cfile
			cd - > /dev/null 2>&1
			;;
		"osmocombb")
			osmoconpid=$(ps -ef|grep 'osmocon' | grep -v 'grep' | awk '{print $2}')
			if [ ${osmoconpid} -gt 0 ] > /dev/null 2>&1
			then
				"${TSHARKBIN}" -i lo -a duration:$T1 -w ${TMPDIR}/tshark_${2}_${4}.pcap > /dev/null 2>&1 &
				"${CCCHBIN}" -s ${L2SOCKET} -a ${2} -i 127.0.0.1 > ${TMPDIR}/ccch_${2}.txt 2>&1 &
				sleep $T1
				disown
				kill -9 $! > /dev/null
			else
				echo "$(date '+%d/%m %H:%M:%S')  Error scaning arfcn ${2}, Osmocon is not running." | tee -a "${APPLOG}"
			fi
			
			;;
		"bladerf")
			"${TSHARKBIN}" -i lo -a duration:$T1 -w ${TMPDIR}/tshark_${2}_${4}.pcap > /dev/null 2>&1 &
			cd ${GSMPATH} > /dev/null 2>&1
			./${GP_BLADE_BIN} -f ${3} -g 70  > ${TMPDIR}/log_airprobe_${2}_${4}.log 2>&1 &
			sleep $T1
			disown
			kill -int $! > /dev/null
			cd - > /dev/null 2>&1
			;;
	esac
}

###########################################################################################################
#
#	Phase 1.2: Analyze PCAP file
#
###########################################################################################################

function extract_cell_data {
#
# $1 Hardware
# $2 pcap  file
# $3 Arfcn
# $4 Airprobe log file
	
	if [ $1 != "osmocombb" ]
	then
		extract_cell_data_no_osmocom $2 $3 $4
	else
		extract_cell_data_osmocom $2 $3
	fi

}

function extract_cell_data_no_osmocom {
#
#	$1: file pcap to be analyzed
#	$2: ARFCN
# 	$3: Airprobe log file
#
	# Numero de abonados
	group_subs=$(cal_num_subs $1 $2)
	
	# IMSIs:
	${TSHARKBIN} -r ${1} -T pdml > /tmp/try_imsis.txt 2>&1
	listado_imsi=$(cat /tmp/try_imsis.txt | grep IMSI | grep "Mobile Identity" | grep -v "unmaskedvalue" | awk -F "IMSI \(" '{print $2}' | cut -d")" -f1)
	rm /tmp/try_imsis.txt
		
	# Numero de frames CCCH capturados
	airprobe_errors=$(grep -c "ERR: conv_decode" "${3}")
	airprobe_decoded=$(grep -c " 0: " "${3}")
	num_lin=$(echo "${airprobe_errors} + ${airprobe_decoded}" | bc )
	umbral_subs=$(echo $[$num_lin * 0,2])

	# Numero de canales y valor de elllos en la celda
	group_chan=$(cal_num_chan $1 $2)

	# Información de Celda
	group_cell=$(cal_cell_info $1 $2)


	num_paged=$(echo ${group_subs} | cut -d"@" -f1)

	num_chan=$(echo $group_chan| cut -d"@" -f1)
	channel=$(echo $group_chan| cut -d"@" -f2)

	cellid=$(echo $group_cell| cut -d"@" -f1)
	lac=$(echo $group_cell| cut -d"@" -f2)
	mcc=$(echo $group_cell| cut -d"@" -f3)
	mnc=$(echo $group_cell| cut -d"@" -f4)
	operador=$(echo $group_cell| cut -d"@" -f5)



	if [ ${num_chan} == "SI1_not_found" ] && [ ${num_paged} != "Page_not_found" ] && [ ${cellid} != "SI3_not_found" ]
	then
		num_chan=0
	fi
	
	if [ ${cellid} != "SI3_not_found" ] && [ ${cellid} != "SI4_found" ] && [ ${num_chan} != "SI1_not_found" ] 2>/dev/null
	then
		cellid_dec=$(echo "ibase=16; ${cellid^^}" | bc)
		lac_dec=$(echo "ibase=16; ${lac^^}" | bc)
				
		if [ $GPSMODE == "variable" ]
		then
				latitude=$(echo ${COORDINATES} | cut -d"," -f1)
				longitude=$(echo ${COORDINATES} | cut -d"," -f2)
		elif [ $GPSMODE == "file" ]
		then
				busca=$(date '+%H%M%S')
				cat "${COORDINATES}" | awk -F"," '/GGA/ {if($7=="1" || $7=="2"){printf "%s@@%s@@%s\n", $2,$3,$5}}' > /tmp/timegps.csv
				
				check=$(grep -c "${busca}" /tmp/timegps.csv)

				if [ ${check} -gt 0 ]
				then
					lat=$(grep ${busca} /tmp/timegps.csv | tail -1 | awk -F"@@" '{print $2}')
					long=$(grep ${busca} /tmp/timegps.csv | tail -1 | awk -F"@@" '{print $3}')
				else
					lat="NA"
					long="NA"
				fi
		else
				latitude="0.0"
				longitude="0.0"
		fi	

		echo "{\"data\": {\"version\": \"0.1.11 - free\", \"latitude\": \"${latitude}\", \"timestamp\": \"$(date '+%Y-%m-%d %H:%M:%S.%6N')\", \"longitude\": \"${longitude}\" }," > "${OUTDIR}"/json.txt
		echo "\"profile\": { \"arfcn\": ${arfcn}, \"cellid\": ${cellid_dec}, \"lac\": ${lac_dec}, \"mcc\": ${mcc}, \"mnc\": ${mnc}, \"numchannel\": ${num_chan}, \"channels\": \"${channel}\"} }," >> "${OUTDIR}"/json.txt

		check_len_imsi=$(echo ${listado_imsi} | wc -c)
		if [ ${check_len_imsi} -gt 10 ]
		then
			echo -n "\"imsidata\": [ " >> "${OUTDIR}"/json.txt
			countimsi=0
			for i in $(echo "${listado_imsi}")
			do
				if [ ${countimsi} -ne 0 ]
				then
					echo -n "," >> "${OUTDIR}"/json.txt
				fi
				echo -n "{\"imsi\": \"${i}\"}" >> "${OUTDIR}"/json.txt
				countimsi=$(echo "${countimsi} + 1" | bc)
			done
			echo " ]," >> "${OUTDIR}"/json.txt
		fi
		
		if [ ${num_lin} -gt 0 ]
		then
			ratio_errors=$(echo "scale=2; ${airprobe_errors} / ${num_lin} * 100" | bc | cut -d"." -f1)
		else
			ratio_errors=666
		fi
		
		sello=$(date '+%d_%m_%H%M%S')
		if [ ${num_lin} -gt 0 ] && [ ${num_chan} != "SI1_not_found" ] && [ ${num_paged} != "Page_not_found" ] && [ ${ratio_errors} -le 30 ]
		then
			if [ ${num_chan} -eq 1 ] && [ ${num_paged} -lt ${umbral_subs} ]
			then
					echo "Alarm, Location Area: ${lac}, CellId: ${cellid}, arfcn: ${arfcn} -> Traffic and Channels: critical situation."
					echo "${sello};${lac};${cellid};${arfcn};${mcc};${mnc};${operador};${num_subs};${num_chan}" >> ${EVENTDIR}/alarms.csv
			elif [ ${num_chan} -gt 1 ] && [ ${num_paged} -lt ${umbral_subs} ]
			then
					echo "Alarm, Location Area: ${lac}, CellId: ${cellid}, arfcn: ${arfcn} -> Low Traffic: review situation."
					echo "${sello};${lac};${cellid};${arfcn};${mcc};${mnc};${operador};${num_subs};${num_chan}" >> ${EVENTDIR}/alarms.csv
			fi
				
			echo "\"traffic\": [{\"numpaged\": ${num_paged}, \"rssi\": ${rxlevel} }] }" >> "${OUTDIR}"/json.txt
			mv "${OUTDIR}"/json.txt "${OUTDIR}"/${sello}_${arfcn}_${cellid}_${lac}_${mcc}_${mnc}.json
			RESULT=1
			
		else
			echo "\"ignored\": [{\"errors\": ${airprobe_errors}, \"decoded\": ${airprobe_decoded}, \"rssi\": ${rxlevel} }] }" >> "${OUTDIR}"/json.txt
			mv "${OUTDIR}"/json.txt "${OUTDIR}"/${sello}_ignored_${arfcn}_${cellid}_${lac}_${mcc}_${mnc}.json
			RESULT=0
		fi
			
	else
			echo "$(date '+%d/%m %H:%M:%S');${arfcn};${cellid};${lac};${mcc};${mnc};${num_chan};${channel};${rxlevel}" >> "${OUTDIR}"/PROFILE_INCOMPLETE.txt
			RESULT=0
	fi
	
	echo ${RESULT}	
}

function extract_cell_data_osmocom {
#
#	$1: file to be analyzed
#	$2: ARFCN
#

	RESULT=0
	# Numero de abonados
	group_subs=$(cal_num_subs "${1}" "${2}" "osmocom")
	
	# IMSIs:
	${TSHARKBIN} -r ${1} -T pdml > /tmp/try_imsis.txt 2>&1
	listado_imsi=$(cat /tmp/try_imsis.txt | grep IMSI | grep "Mobile Identity" | grep -v "unmaskedvalue" | awk -F "IMSI \(" '{print $2}' | cut -d")" -f1)
	rm /tmp/try_imsis.txt

	# Numero de frames CCCH capturados
	num_lin=$(wc -l ${TMPDIR}/ccch_${2}.txt | awk '{print $1}')
	umbral_subs=$(echo $[$num_lin * 0,2])

	# Numero de canales y valor de elllos en la celda
	group_chan=$(cal_num_chan "${1}" "${2}" "osmocom")

	# Información de Celda
	group_cell=$(cal_cell_info "${1}" "${2}" "osmocom")

	num_paged=$(echo ${group_subs} | cut -d"@" -f1)

	num_burst=$(grep BURST ${TMPDIR}/ccch_${arfcn}.txt | wc -l)
	num_drop=$(grep Dropping ${TMPDIR}/ccch_${arfcn}.txt | wc -l)
	num_fbsb=$(grep FBSB ${TMPDIR}/ccch_${arfcn}.txt | wc -l)
	errors=$(echo "$num_drop + $num_fbsb + $num_burst" | bc)
	rm ${TMPDIR}/ccch_${2}.txt
	
	umbral=$(echo $[$num_lin/2])
	umbral_fb=$(echo "$num_lin / 1.55"| bc)
	umbral_subs=$(echo $[$num_lin * 0,2])
	
	num_chan=$(echo $group_chan| cut -d"@" -f1)
	channel=$(echo $group_chan| cut -d"@" -f2)

	cellid=$(echo $group_cell| cut -d"@" -f1)
	lac=$(echo $group_cell| cut -d"@" -f2)
	mcc=$(echo $group_cell| cut -d"@" -f3)
	mnc=$(echo $group_cell| cut -d"@" -f4)
	operador=$(echo $group_cell| cut -d"@" -f5)
		
	if [ ${cellid} != "SI3_not_found" ] && [ ${cellid} != "SI4_found" ] && [ ${num_chan} != "SI1_not_found" ] 2>/dev/null
	then
	
		cellid_dec=$(echo "ibase=16; ${cellid^^}" | bc)
		lac_dec=$(echo "ibase=16; ${lac^^}" | bc)
				
		if [ ${GPSMODE} == "variable" ]
		then
				latitude=$(echo ${COORDINATES} | cut -d"," -f1)
				longitude=$(echo ${COORDINATES} | cut -d"," -f2)
		elif [ ${GPSMODE} == "file" ]
		then
				busca=$(date '+%H%M%S')
				cat "${COORDINATES}" | awk -F"," '/GGA/ {if($7=="1" || $7=="2"){printf "%s@@%s@@%s\n", $2,$3,$5}}' > /tmp/timegps.csv
				
				check=$(grep -c "${busca}" /tmp/timegps.csv)

				if [ ${check} -gt 0 ]
				then
					lat=$(grep ${busca} /tmp/timegps.csv | tail -1 | awk -F"@@" '{print $2}')
					long=$(grep ${busca} /tmp/timegps.csv | tail -1 | awk -F"@@" '{print $3}')
				else
					lat="NA"
					long="NA"
				fi
		else
				latitude="0.0"
				longitude="0.0"
		fi
		
		echo "{\"data\": {\"version\": \"0.1.11 - free\", \"latitude\": \"${latitude}\", \"timestamp\": \"$(date '+%Y-%m-%d %H:%M:%S.%N')\", \"longitude\": \"${longitude}\" }," > "${OUTDIR}"/json.txt
		echo "\"profile\": { \"arfcn\": ${arfcn}, \"cellid\": ${cellid_dec}, \"lac\": ${lac_dec}, \"mcc\": ${mcc}, \"mnc\": ${mnc}, \"numchannel\": ${num_chan}, \"channels\": \"${channel}\" }," >> "${OUTDIR}"/json.txt
		
		check_len_imsi=$(echo ${listado_imsi} | wc -c)
		if [ ${check_len_imsi} -gt 10 ]
		then
			echo -n "\"imsidata\": [ " >> "${OUTDIR}"/json.txt
			countimsi=0
			for i in $(echo "${listado_imsi}")
			do
				if [ ${countimsi} -ne 0 ]
				then
					echo -n "," >> "${OUTDIR}"/json.txt
				fi
				echo -n "{\"imsi\": \"${i}\"}" >> "${OUTDIR}"/json.txt
				countimsi=$(echo "${countimsi} + 1" | bc)
			done
			echo " ]," >> "${OUTDIR}"/json.txt
		fi
		
		sello=$(date '+%d_%m_%H%M%S')
		if [ ${num_burst} -lt ${umbral} ] && [ ${num_drop} -lt ${umbral} ]  && [ ${num_fbsb} -lt ${umbral_fb} ]
		then
			if [ ${num_chan} -eq 1 ] && [ ${num_paged} -lt ${umbral_subs} ]
            then
					echo "Alarm, Location Area: ${lac}, CellId: ${cellid}, arfcn: ${arfcn} -> Traffic and Channels: critical situation."
					echo "${sello};${lac};${cellid};${arfcn};${mcc};${mnc};${operador};${num_subs};${num_chan}" >> ${EVENTDIR}/alarms.csv
			elif [ ${num_chan} -gt 1 ] && [ ${num_paged} -lt ${umbral_subs} ]
			then
					echo "Alarm, Location Area: ${lac}, CellId: ${cellid}, arfcn: ${arfcn} -> Low Traffic: review situation."
					echo "${sello};${lac};${cellid};${arfcn};${mcc};${mnc};${operador};${num_subs};${num_chan}" >> ${EVENTDIR}/alarms.csv
			fi
				
			echo "\"traffic\": [\"numpaged\": ${num_paged}, \"rssi\": ${rxlevel} }] }" >> "${OUTDIR}"/json.txt
			mv "${OUTDIR}"/json.txt "${OUTDIR}"/${sello}_${arfcn}_${cellid}_${lac}_${mcc}_${mnc}.json				
			RESULT=1
						
		else
			echo "\"ignored\": [{\"errors\": ${errors}, \"decoded\": ${num_lin}}] }" >> "${OUTDIR}"/json.txt
			mv "${OUTDIR}"/json.txt "${OUTDIR}"/${sello}_ignored_${arfcn}_${cellid}_${lac}_${mcc}_${mnc}.json
			RESULT=0
		fi
					
	else
		echo "$(date '+%d/%m %H:%M:%S');${arfcn};${cellid_dec};${lac_dec};${mcc};${mnc};${num_chan};${channel};${rxlevel}" >> "${OUTDIR}"/PROFILE_INCOMPLETE.txt
		RESULT=0
	fi
	
	echo ${RESULT}	
}

###########################################################################################################
#
#	Finally; Execution
#
###########################################################################################################

disclaimer

if [ $# -eq 0 ]
then
	usage	
	exit 0
else
	while getopts "hw:lg" opt; do
	  case $opt in
	    w)
	      HARDWARE="${OPTARG}"
	      case "${HARDWARE}" in
		"osmocombb");;
		"usrp");;
		"rtlsdr");;
		"rtlsdr-RPi");;
		"bladerf");;
		"modem");;
		*)
		 echo "Bad parameter, "${OPTARG}""
		 usage
		 exit 0
	      esac
	      echo "Using hardware: $OPTARG" >&2 | tee -a "${APPLOG}"
	      ;;
	     l)
	      LOOP="TRUE"
	      echo "Warning !!   Using infinite loop mode."
	      ;;
	     g)
	      GPSMODE="${OPTARG}"
	      case "${GPSMODE}" in
		 "variable");;
		 "file");;
			*)
			echo "Bad parameter, "${OPTARG}", -g option can only be used with file or variable values."
			usage
			exit 0
		 esac
		 echo "Using second hardware: $HARDWARE2" >&2 | tee -a "${APPLOG}"
	      echo "Using GPS"
	      ;;
	    \?)
	      echo "Invalid option: -$OPTARG" >&2 
	      usage
	      exit 1
	      ;;
	    h)
	      usage
	      exit 0
	      ;;
	    *)
	      echo "Option -$OPTARG requires an argument." >&2
	      exit 1
	      ;;
	  esac
	done
fi


check_bin "${HARDWARE}"

if [ ${LOOP} == "FALSE" ]
then

	find_close_cells "${HARDWARE}"

	scan_cells "${HARDWARE}"

elif [ ${LOOP} == "TRUE" ]
then

	while true
	do
		find_close_cells "${HARDWARE}"

		scan_cells "${HARDWARE}"

		sleep 5
	done
fi

echo "$(date +'%d/%m %H:%M:%S')  Script finished." | tee -a "${APPLOG}"
